@@ -102,7 +102,7 @@ env = {
     "platform_version": platform.version(),
     "python_full_version": platform.python_version(),
     "platform_python_implementation": platform.python_implementation(),
-    "python_version": platform.python_version()[:3],
+    "python_version": ".".join(platform.python_version_tuple()[:2]),
     "sys_platform": sys.platform,
     "version_info": tuple(sys.version_info),
     # Extra information
@@ -145,6 +145,38 @@ import sysconfig
 print(json.dumps(sysconfig.get_paths()))
 """
 
+GET_PATHS_FOR_GENERIC_ENVS = """\
+# We can't use sysconfig.get_paths() because
+# on some distributions it does not return the proper paths
+# (those used by pip for instance). We go through distutils
+# to get the proper ones.
+import json
+import site
+import sysconfig
+
+from distutils.command.install import SCHEME_KEYS  # noqa
+from distutils.core import Distribution
+
+d = Distribution()
+d.parse_config_files()
+obj = d.get_command_obj("install", create=True)
+obj.finalize_options()
+
+paths = sysconfig.get_paths().copy()
+for key in SCHEME_KEYS:
+    if key == "headers":
+        # headers is not a path returned by sysconfig.get_paths()
+        continue
+
+    paths[key] = getattr(obj, f"install_{key}")
+
+if site.check_enableusersite() and hasattr(obj, "install_usersite"):
+    paths["usersite"] = getattr(obj, "install_usersite")
+    paths["userbase"] = getattr(obj, "install_userbase")
+
+print(json.dumps(paths))
+"""
+
 
 class SitePackages:
     def __init__(
@@ -615,7 +647,7 @@ class EnvManager(object):
 
         self.remove_venv(venv)
 
-        return VirtualEnv(venv)
+        return VirtualEnv(venv, venv)
 
     def create_venv(
         self, io, name=None, executable=None, force=False
@@ -753,7 +785,7 @@ class EnvManager(object):
                     "</>"
                 )
 
-                return SystemEnv(Path(sys.prefix))
+                return self.get_system_env()
 
             io.write_line(
                 "Creating virtualenv <c1>{}</> in {}".format(name, str(venv_path))
@@ -834,7 +866,9 @@ class EnvManager(object):
                 shutil.rmtree(str(file_path))
 
     @classmethod
-    def get_system_env(cls, naive=False):  # type: (bool) -> "SystemEnv"
+    def get_system_env(
+        cls, naive=False
+    ):  # type: (bool) -> Union["SystemEnv", "GenericEnv"]
         """
         Retrieve the current Python environment.
         This can be the base Python environment or an activated virtual environment.
@@ -846,17 +880,21 @@ class EnvManager(object):
         (e.g. plugin installation or self update).
         """
         prefix, base_prefix = Path(sys.prefix), Path(cls.get_base_prefix())
-        if naive is False:
-            from poetry.locations import data_dir
-
-            try:
-                prefix.relative_to(data_dir())
-            except ValueError:
-                pass
+        env = SystemEnv(prefix)
+        if not naive:
+            if prefix.joinpath("poetry_env").exists():
+                env = GenericEnv(base_prefix, child_env=env)
             else:
-                prefix = base_prefix
+                from poetry.locations import data_dir
 
-        return SystemEnv(prefix)
+                try:
+                    prefix.relative_to(data_dir())
+                except ValueError:
+                    pass
+                else:
+                    env = GenericEnv(base_prefix, child_env=env)
+
+        return env
 
     @classmethod
     def get_base_prefix(cls):  # type: () -> str
@@ -885,13 +923,24 @@ class Env(object):
 
     def __init__(self, path, base=None):  # type: (Path, Optional[Path]) -> None
         self._is_windows = sys.platform == "win32"
+        self._is_mingw = sysconfig.get_platform().startswith("mingw")
+        self._is_conda = bool(os.environ.get("CONDA_DEFAULT_ENV"))
+
+        if not self._is_windows or self._is_mingw:
+            bin_dir = "bin"
+        else:
+            bin_dir = "Scripts"
 
         self._path = path
-        bin_dir = "bin" if not self._is_windows else "Scripts"
         self._bin_dir = self._path / bin_dir
 
         self._base = base or path
 
+        self._executable = "python"
+        self._pip_executable = "pip"
+
+        self.find_executables()
+
         self._marker_env = None
         self._pip_version = None
         self._site_packages = None
@@ -922,7 +971,7 @@ class Env(object):
         """
         Path to current python executable
         """
-        return self._bin("python")
+        return self._bin(self._executable)
 
     @property
     def marker_env(self):
@@ -931,12 +980,16 @@ class Env(object):
 
         return self._marker_env
 
+    @property
+    def parent_env(self):  # type: () -> GenericEnv
+        return GenericEnv(self.base, child_env=self)
+
     @property
     def pip(self):  # type: () -> str
         """
         Path to current pip executable
         """
-        return self._bin("pip")
+        return self._bin(self._pip_executable)
 
     @property
     def platform(self):  # type: () -> str
@@ -1028,6 +1081,41 @@ class Env(object):
 
         return sys.prefix
 
+    def _find_python_executable(self):  # type: () -> None
+        bin_dir = self._bin_dir
+
+        if self._is_windows and self._is_conda:
+            bin_dir = self._path
+
+        python_executables = sorted(
+            p.name
+            for p in bin_dir.glob("python*")
+            if re.match(r"python(?:\d+(?:\.\d+)?)?(?:\.exe)?$", p.name)
+        )
+        if python_executables:
+            executable = python_executables[0]
+            if executable.endswith(".exe"):
+                executable = executable[:-4]
+
+            self._executable = executable
+
+    def _find_pip_executable(self):  # type: () -> None
+        pip_executables = sorted(
+            p.name
+            for p in self._bin_dir.glob("pip*")
+            if re.match(r"pip(?:\d+(?:\.\d+)?)?(?:\.exe)?$", p.name)
+        )
+        if pip_executables:
+            pip_executable = pip_executables[0]
+            if pip_executable.endswith(".exe"):
+                pip_executable = pip_executable[:-4]
+
+            self._pip_executable = pip_executable
+
+    def find_executables(self):  # type: () -> None
+        self._find_python_executable()
+        self._find_pip_executable()
+
     def get_version_info(self):  # type: () -> Tuple[int]
         raise NotImplementedError()
 
@@ -1063,11 +1151,17 @@ class Env(object):
         cmd = [bin] + list(args)
         return self._run(cmd, **kwargs)
 
+    def run_python(self, *args, **kwargs):
+        return self.run(self._executable, *args, **kwargs)
+
     def run_pip(self, *args, **kwargs):
         pip = self.get_pip_command()
         cmd = pip + list(args)
         return self._run(cmd, **kwargs)
 
+    def run_python_script(self, content, **kwargs):  # type: (str, Any) -> str
+        return self.run(self._executable, "-W", "ignore", "-", input_=content, **kwargs)
+
     def _run(self, cmd, **kwargs):
         """
         Run a command inside the Python environment.
@@ -1133,7 +1227,11 @@ class Env(object):
         """
         Return path to the given executable.
         """
-        bin_path = (self._bin_dir / bin).with_suffix(".exe" if self._is_windows else "")
+        if self._is_windows and not bin.endswith(".exe"):
+            bin_path = self._bin_dir / (bin + ".exe")
+        else:
+            bin_path = self._bin_dir / bin
+
         if not bin_path.exists():
             # On Windows, some executables can be in the base path
             # This is especially true when installing Python with
@@ -1144,7 +1242,11 @@ class Env(object):
             # that creates a fake virtual environment pointing to
             # a base Python install.
             if self._is_windows:
-                bin_path = (self._path / bin).with_suffix(".exe")
+                if not bin.endswith(".exe"):
+                    bin_path = self._path / (bin + ".exe")
+                else:
+                    bin_path = self._path / bin
+
                 if bin_path.exists():
                     return str(bin_path)
 
@@ -1270,16 +1372,16 @@ class VirtualEnv(Env):
         # In this case we need to get sys.base_prefix
         # from inside the virtualenv.
         if base is None:
-            self._base = Path(self.run("python", "-", input_=GET_BASE_PREFIX).strip())
+            self._base = Path(self.run_python_script(GET_BASE_PREFIX).strip())
 
     @property
     def sys_path(self):  # type: () -> List[str]
-        output = self.run("python", "-", input_=GET_SYS_PATH)
+        output = self.run_python_script(GET_SYS_PATH)
 
         return json.loads(output)
 
     def get_version_info(self):  # type: () -> Tuple[int]
-        output = self.run("python", "-", input_=GET_PYTHON_VERSION)
+        output = self.run_python_script(GET_PYTHON_VERSION)
 
         return tuple([int(s) for s in output.strip().split(".")])
 
@@ -1289,7 +1391,7 @@ class VirtualEnv(Env):
     def get_pip_command(self):  # type: () -> List[str]
         # We're in a virtualenv that is known to be sane,
         # so assume that we have a functional pip
-        return [self._bin("pip")]
+        return [self._bin(self._pip_executable)]
 
     def get_supported_tags(self):  # type: () -> List[Tag]
         file_path = Path(packaging.tags.__file__)
@@ -1308,6 +1410,10 @@ class VirtualEnv(Env):
             "from ._typing import MYPY_CHECK_RUNNING, cast",
             "MYPY_CHECK_RUNNING = False\ncast = lambda type_, value: value",
         )
+        script = script.replace(
+            "from . import _manylinux, _musllinux",
+            "from pip._vendor.packaging import _manylinux, _musllinux",
+        )
 
         script += textwrap.dedent(
             """
@@ -1317,12 +1423,12 @@ class VirtualEnv(Env):
             """
         )
 
-        output = self.run("python", "-", input_=script)
+        output = self.run_python_script(script)
 
         return [Tag(*t) for t in json.loads(output)]
 
     def get_marker_env(self):  # type: () -> Dict[str, Any]
-        output = self.run("python", "-", input_=GET_ENVIRONMENT_INFO)
+        output = self.run(self._executable, "-", input_=GET_ENVIRONMENT_INFO)
 
         return json.loads(output)
 
@@ -1335,7 +1441,7 @@ class VirtualEnv(Env):
         return Version.parse(m.group(1))
 
     def get_paths(self):  # type: () -> Dict[str, str]
-        output = self.run("python", "-", input_=GET_PATHS)
+        output = self.run_python_script(GET_PATHS)
 
         return json.loads(output)
 
@@ -1387,6 +1493,86 @@ class VirtualEnv(Env):
         return os.pathsep.join([str(self._bin_dir), os.environ.get("PATH", "")])
 
 
+class GenericEnv(VirtualEnv):
+    def __init__(
+        self, path, base=None, child_env=None
+    ):  # type: (Path, Optional[Path], Optional[Env]) -> None
+        self._child_env = child_env
+
+        super(GenericEnv, self).__init__(path, base=base)
+
+    def find_executables(self):  # type: () -> None
+        patterns = [("python*", "pip*")]
+
+        if self._child_env:
+            minor_version = "{}.{}".format(
+                self._child_env.version_info[0], self._child_env.version_info[1]
+            )
+            major_version = "{}".format(self._child_env.version_info[0])
+            patterns = [
+                ("python{}".format(minor_version), "pip{}".format(minor_version)),
+                ("python{}".format(major_version), "pip{}".format(major_version)),
+            ]
+
+        python_executable = None
+        pip_executable = None
+
+        for python_pattern, pip_pattern in patterns:
+            if python_executable and pip_executable:
+                break
+
+            if not python_executable:
+                python_executables = sorted(
+                    [
+                        p.name
+                        for p in self._bin_dir.glob(python_pattern)
+                        if re.match(r"python(?:\d+(?:\.\d+)?)?(?:\.exe)?$", p.name)
+                    ]
+                )
+
+                if python_executables:
+                    executable = python_executables[0]
+                    if executable.endswith(".exe"):
+                        executable = executable[:-4]
+
+                    python_executable = executable
+
+            if not pip_executable:
+                pip_executables = sorted(
+                    [
+                        p.name
+                        for p in self._bin_dir.glob(pip_pattern)
+                        if re.match(r"pip(?:\d+(?:\.\d+)?)?(?:\.exe)?$", p.name)
+                    ]
+                )
+                if pip_executables:
+                    pip_executable = pip_executables[0]
+                    if pip_executable.endswith(".exe"):
+                        pip_executable = pip_executable[:-4]
+
+                    pip_executable = pip_executable
+
+            if python_executable:
+                self._executable = python_executable
+
+            if pip_executable:
+                self._pip_executable = pip_executable
+
+    def get_paths(self):  # type: () -> Dict[str, str]
+        output = self.run_python_script(GET_PATHS_FOR_GENERIC_ENVS)
+
+        return json.loads(output)
+
+    def execute(self, bin, *args, **kwargs):  # type: (str, str, Any) -> Optional[int]
+        return super(VirtualEnv, self).execute(bin, *args, **kwargs)
+
+    def _run(self, cmd, **kwargs):  # type: (List[str], Any) -> Optional[int]
+        return super(VirtualEnv, self)._run(cmd, **kwargs)
+
+    def is_venv(self):  # type: () -> bool
+        return self._path != self._base
+
+
 class NullEnv(SystemEnv):
     def __init__(self, path=None, base=None, execute=False):
         if path is None:
